﻿using System;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System.Xml;

public class CustomMessageEncoderBindingElement : MessageEncodingBindingElement
{
	readonly CustomMessageEncoderBindingElement baseBindingElement;
	readonly MessageEncodingBindingElement innerBindingElement;

	public CustomMessageEncoderBindingElement(MessageEncodingBindingElement innerBindingElement)
	   : base()
	{
		this.innerBindingElement = innerBindingElement;
	}

	CustomMessageEncoderBindingElement(CustomMessageEncoderBindingElement originalBindingElement)
	   : this(originalBindingElement.innerBindingElement)
	{
		if (originalBindingElement.baseBindingElement == null)
		{
			this.baseBindingElement = originalBindingElement;
		}
		else
		{
			this.baseBindingElement = originalBindingElement.baseBindingElement;
		}
	}

	public override MessageVersion MessageVersion
	{
		get
		{
			return this.innerBindingElement.MessageVersion;
		}

		set
		{
			if (value == null)
				throw new ArgumentNullException("value");
			this.innerBindingElement.MessageVersion = value;
		}
	}

	public override IChannelFactory<TChannel> BuildChannelFactory<TChannel>(BindingContext context)
	{
		if (context == null)
			throw new ArgumentNullException("context");

		context.BindingParameters.Add(this);
		return context.BuildInnerChannelFactory<TChannel>();
	}

	public override bool CanBuildChannelFactory<TChannel>(BindingContext context)
	{
		if (context == null)
			throw new ArgumentNullException("context");

		return context.CanBuildInnerChannelFactory<TChannel>();
	}

	public override IChannelListener<TChannel> BuildChannelListener<TChannel>(BindingContext context)
	{
		if (context == null)
			throw new ArgumentNullException("context");

		context.BindingParameters.Add(this);
		return context.BuildInnerChannelListener<TChannel>();
	}

	public override bool CanBuildChannelListener<TChannel>(BindingContext context)
	{
		if (context == null)
			throw new ArgumentNullException("context");

		context.BindingParameters.Add(this);
		return context.CanBuildInnerChannelListener<TChannel>();
	}

	public override MessageEncoderFactory CreateMessageEncoderFactory()
	{
		return new CustomMessageEncoderFactory(this, innerBindingElement.CreateMessageEncoderFactory());
	}

	public override BindingElement Clone()
	{
		return new CustomMessageEncoderBindingElement(this);
	}

	public override T GetProperty<T>(BindingContext context)
	{
		return innerBindingElement.GetProperty<T>(context) ?? context.GetInnerProperty<T>();
	}

	CustomMessageEncoderBindingElement BaseBindingElement
	{
		get { return this.baseBindingElement ?? this; }
	}
}